from argparse import ArgumentParser
from pathlib import Path
from random import choice
from shutil import move
from time import sleep

from htsct.utils.tools import search


class CLICommand:
    """文件自动移动

    移动指定文件夹下的文件到当前文件夹并等待对应程序消耗文件后自动补充
    """

    @staticmethod
    def add_argments(parser: ArgumentParser):
        parser.add_argument("--folder_num", nargs=1, default=1, help="自动管理的文件夹数量", dest="folder_num", type=int)
        parser.add_argument("-s", nargs=1, required=True, help="资源文件夹", dest="src_folder", type=str)
        parser.add_argument("--max_file", nargs=1, default=1, help="单次最大排队文件数量", dest="max_file", type=int)
        parser.add_argument("-t", nargs=1, default=".cif", help="自动管理的文件类型", dest="file_type", type=str)

    @staticmethod
    def run(args, parser):
        task_folders = [str(i) for i in range(1, args.folder_num[0] + 1)]
        for task_folder in task_folders:
            Path(task_folder).mkdir(exist_ok=True)
        task_manager(args.file_type[0], args.src_folder[0], task_folders, args.max_file[0])


def __task_manager(ftype, src_folder, task_folders: list, max_file):
    while True:
        src_files = list(search(src_folder, ftype))
        if src_files:
            for task_folder in task_folders:
                task_files = search(task_folder, ftype)
                length = len(list(task_files))
                if length < max_file:
                    src_file = choice(src_files)
                    move(src_file, task_folder)
                    break
        sleep(1)


def task_manager(ftype, src_folder, task_folders: list, max_file=3):
    """用于自动移动指定文件类型到指定目录
    :param ftype: 文件类型(后缀)
    :param src_folder: 源文件夹
    :param task_folders: 目标文件夹列表
    :param max_file: 最大移动数目
    """
    from threading import Thread
    t1 = Thread(target=__task_manager, args=[ftype, src_folder, task_folders, max_file])
    t1.start()
