from os.path import expanduser
from pathlib import Path
from pathlib import PurePath
from typing import IO, TextIO
from typing import Union

from toml import load

from htsct import statics
from htsct.utils.resources import files as packageFiles

NameOrFile = Union[str, PurePath, IO]
HTS_HOME = Path(expanduser('~')) / "htsct"
STATIC_PATH = Path(statics.__path__[0])
# vasp path
VASP_STD = ""
VASP_NCL = ""
VASP_GAM = ""
VASP_INPUTS = ""

# script path
QSUB = ""

# OUTPUT
TASK_LOG = ""
FIGS_PATH = ""
DB_DIR = ""
WORK_PATH = ""
SYSTEM_LOG = HTS_HOME / "log.txt"
CALC_FILES = HTS_HOME / "calcs"

# settings
WAIT_INTERVAL = 0  # 空闲等待时间
INPUT_FILE_TYPE = ""


# load from files
def load_config(fd: TextIO):
    data = load(fd)
    config = {}
    for _ in data.values():
        config.update({k.upper(): v for k, v in _.items()})
    globals().update(config)


if not (HTS_HOME / ".htsct.toml").is_file():
    with open(packageFiles('htsct.statics') / ".htsct.toml") as fd:
        load_config(fd)
else:
    with open(HTS_HOME / ".htsct.toml") as fd:
        load_config(fd)

if not (HTS_HOME / ".htsct.sub.sh").is_file():
    QSUB = STATIC_PATH / ".htsct.sub.sh"

if not (HTS_HOME / ".guiConfig.toml").is_file():
    GUI_CONFIG = STATIC_PATH / ".guiConfig.toml"
else:
    GUI_CONFIG = HTS_HOME / ".guiConfig.toml"

if not (HTS_HOME / "vaspInputs.toml").is_file():
    VASP_INPUTS = STATIC_PATH / "vaspInputs.toml"
else:
    VASP_INPUTS = HTS_HOME / "vaspInputs.toml"
