import sqlite3
from typing import List, Dict


def is_exist(conn, table_name="ribbon"):
    """
    Determine whether the table exists.
    :param conn: Instance of sqlite3.Connection
    :param table_name: table name
    :return: boolean
    """
    sql = "SELECT name FROM sqlite_master WHERE type='table' ORDER BY name"
    cur = conn.cursor()
    cur.execute(sql)
    res = cur.fetchall()
    tables = [i[0] for i in res]
    cur.close()
    if table_name in tables:
        return True
    return False


def create_table(conn, table_name, dic):
    """
    Create a table from keys of a dict.
    :param conn:Instance of sqlite3.Connection
    :param table_name: table name
    :param dic: dict object
    """
    keys = list(map(lambda x: f"`{x}`", dic.keys()))
    sql = f'create table {table_name}(\'row_id\' integer primary key autoincrement,' + ' varchar,'.join(
        keys) + ' varchar)'
    cur = conn.cursor()
    cur.execute(sql)
    cur.close()
    conn.commit()


def show_table_field(conn, table_name):
    """
    Show table fields.
    :param conn:Instance of sqlite3.Connection
    :param table_name:table name
    :return: table fields
    """
    sql = f"PRAGMA table_info({table_name})"
    cur = conn.cursor()
    cur.execute(sql)
    res = cur.fetchall()
    cur.close()
    return [i[1] for i in res]


def add_field(conn, table_name, fields: List):
    """
    Add fields to table.
    :param conn: Instance of sqlite3.Connection
    :param table_name:table name
    :param fields: a list of table fields
    :return:
    """
    cur = conn.cursor()
    for field in fields:
        field = field.replace("'", "''")
        sql = f"ALTER TABLE {table_name} ADD COLUMN '{field}' varchar"
        cur.execute(sql)
    cur.close()
    conn.commit()


def translate(s):
    return str(s).replace("'", "''")


def write_db(conn: sqlite3.Connection, table_name: str, dic: Dict):
    """
    Insert dict to table.
    :param conn:Instance of sqlite3.Connection
    :param table_name:table name
    :param dic:dict object
    """
    if not is_exist(conn, table_name):
        create_table(conn, table_name, dic)
    fields = show_table_field(conn, table_name)
    dic2 = map(lambda x: str(x), dic)
    new_fields = list(set(dic2) - set(fields))
    if new_fields:
        add_field(conn, table_name, new_fields)
    cur = conn.cursor()
    keys = dic.keys()
    values = []
    for k in keys:
        values.append(str(dic[k]))
    values = map(lambda x: x.replace("'", "''"), values)
    table_keys = "`" + "`,`".join([str(k) for k in keys]) + "`"
    string = "'" + "','".join(values) + "'"
    sql = f"insert into {table_name}({table_keys}) values({string})"
    cur.execute(sql)
    conn.commit()


def update_db_by_id(conn: sqlite3.Connection, table_name: str, dic: Dict, unique_id=""):
    """
        Insert dict to table.
        :param unique_id:
        :param conn:Instance of sqlite3.Connection
        :param table_name:table name
        :param dic:dict object
        """
    if not is_exist(conn, table_name):
        create_table(conn, table_name, dic)
    fields = show_table_field(conn, table_name)
    dic2 = map(lambda x: str(x), dic)
    new_fields = list(set(dic2) - set(fields))
    if new_fields:
        add_field(conn, table_name, new_fields)
    cur = conn.cursor()
    keys = dic.keys()
    values = []
    for k in keys:
        values.append(str(dic[k]))
    values = list(map(lambda x: x.replace("'", "''"), values))
    kv = ",".join([f"`{k}`='{v}'" for k, v in zip(keys, values)])
    sql = f"update {table_name} set {kv} where `unique_id` = '{unique_id}'"
    cur.execute(sql)
    if cur.rowcount < 1:
        print("erro!")
    elif cur.rowcount > 1:
        print("affect muti-rows!!")
    conn.commit()


def is_id_exist(conn: sqlite3.Connection, table_name: str, unique_id=""):
    cur = conn.cursor()
    sql = f"select `unique_id` from {table_name} where `unique_id` = '{unique_id}'"
    cur.execute(sql)
    unique_ids = cur.fetchone()
    cur.close()
    if unique_ids:
        return True


if __name__ == '__main__':
    d1 = {"A": 100, "B": 200, 23: "xxx", "unique_id": "zhaohao"}
    d2 = {"A": 10, "Y": "asf", "B": 200, 23: "xxx", "C": 90, "D": 234, "asf": "d'f'", "fasf'sdgf'": 32945,
          "sfaaf": 324,
          }
    conn = sqlite3.connect("test.db")
    # write_db(conn, "ribbon1", d1)
    # update_db_by_id(conn, "ribbon1", d2, unique_id="zhaohao")
    print(is_id_exist(conn, "ribbon1", unique_id="zhaohao"))
    conn.close()
